@@ -1,4 +1,4 @@
-match<-function(z,fine=rep(1,length(z)),dist,dat,ncontrol=1,penalty=round(max(dist$d)*10000),s.cost=100,subX=NULL){
+match<-function(z,p,dist,dat,exact=NULL,fine=rep(1,length(z)),ncontrol=1,penalty=round(max(dist$d)*1000),s.cost=100,subX=NULL){
   #Check input
   stopifnot(is.data.frame(dat))
   stopifnot(is.vector(z))
@@ -26,8 +26,20 @@ match<-function(z,fine=rep(1,length(z)),dist,dat,ncontrol=1,penalty=round(max(di
     stopifnot(is.vector(subX))
   }
 
+  #sort input
+  if (is.null(exact)){
+    o<-order(1-p)
+  }else{
+    o<-order(exact,1-p)
+    exact<-exact[o]
+  }
+
+  z<-z[o]
+  p<-p[o]
+  fine<-fine[o]
+  dat<-dat[o,]
+
   #Must have treated first
-  n<-length(z)
   if(!(min(z[1:(n-1)]-z[2:n])>=0)){
     o<-order(1-z)
     z<-z[o]
@@ -38,12 +50,37 @@ match<-function(z,fine=rep(1,length(z)),dist,dat,ncontrol=1,penalty=round(max(di
 
   #do match
   if (!requireNamespace("optmatch", quietly=TRUE)) {
-    stop("Error: package optmatch (>= 0.9-1) not loaded.  To run rcbalance command, you must install optmatch first and agree to the terms of its license.")
+    stop("Error: package optmatch (>= 0.9-1) not loaded.  To run match command, you must install optmatch first and agree to the terms of its license.")
   }
 
   net<-net(z,dist,ncontrol,fine,penalty,s.cost,subX)
   if (any(net$cost==Inf)) net$cost[net$cost==Inf]<-2*max(net$cost[net$cost!=Inf])
-  output<-rcbalance::callrelax(net)
+
+  callrelax <- function (net) {
+    if (!requireNamespace("optmatch", quietly = TRUE)) {
+      stop('Error: package optmatch (>= 0.9-1) not loaded.  To run rcbalance command, you must install optmatch first and agree to the terms of its license.')
+    }
+    startn <- net$startn
+    endn <- net$endn
+    ucap <- net$ucap
+    b <- net$b
+    cost <- net$cost
+    nnodes <- length(b)
+    my.expr <- parse(text = '.Fortran("relaxalg", nnodes, as.integer(length(startn)),
+                     as.integer(startn), as.integer(endn), as.integer(cost),
+                     as.integer(ucap), as.integer(b), x1 = integer(length(startn)),
+                     crash1 = as.integer(0), large1 = as.integer(.Machine$integer.max/4),
+                     feasible1 = integer(1), NAOK = FALSE, DUP = TRUE, PACKAGE = "optmatch")')
+    fop <- eval(my.expr)
+    x <- fop$x1
+    feasible <- fop$feasible1
+    crash <- fop$crash1
+    list(crash = crash, feasible = feasible, x = x)
+  }
+
+  #output<-rcbalance::callrelax(net)
+  output<-callrelax(net)
+
   if (output$feasible!=1){
     warning("Match is infeasible.  Change dist or ncontrol to obtain a feasible match.")
     m<-list(feasible=output$feasible,d=NULL)
@@ -51,15 +88,12 @@ match<-function(z,fine=rep(1,length(z)),dist,dat,ncontrol=1,penalty=round(max(di
     x<-output$x[1:net$tcarcs]
     treated<-net$startn[1:net$tcarcs]
     control<-net$endn[1:net$tcarcs]
-    match.df<-data.frame('treat' = treated, 'x' = x, 'control' = control)
-    matched.or.not<-plyr::daply(match.df, plyr::.(match.df$treat),
-                                function(treat.edges) c(as.numeric(as.character(treat.edges$treat[1])), sum(treat.edges$x)), .drop_o = FALSE)
-    if(any(matched.or.not[,2] == 0)){
-      match.df<-match.df[-which(match.df$treat %in% matched.or.not[which(matched.or.not[,2] == 0),1]),]
-    }
+    treated=treated[which(x==1)]
+    control=control[which(x==1)]
+    match.df=data.frame('treat'=treated,'control'=control)
     match.df$treat<-as.factor(as.character(match.df$treat))
     matches<-as.matrix(plyr::daply(match.df, plyr::.(match.df$treat),
-                                   function(treat.edges) treat.edges$control[treat.edges$x == 1], .drop_o = FALSE))
+                                   function(treat.edges) treat.edges$control,.drop_o=FALSE))
     id1<-(1:n)[z==1]
     id0<-(1:n)[z==0]
     matchid<-matrix(c(id1[as.numeric(row.names(matches))], id0[as.vector((matches-sum(z)))]),ncol=ncontrol+1)
